@@ -17,10 +17,11 @@
 # Written by:
 #
 #   Marvin N. Wright
-# Institut für Medizinische Biometrie und Statistik
-# Universität zu Lübeck
+# Institut fuer Medizinische Biometrie und Statistik
+# Universitaet zu Luebeck
 # Ratzeburger Allee 160
-# 23562 Lübeck
+# 23562 Luebeck
+# Germany
 #
 # http://www.imbs-luebeck.de
 # wright@imbs.uni-luebeck.de
@@ -46,6 +47,7 @@ print.ranger <- function(x, ...) {
   cat("Target node size:                ", x$min.node.size, "\n")
   cat("Variable importance mode:        ", x$importance.mode, "\n")
   if (x$treetype == "Survival") {
+    cat("Splitrule:                       ", x$splitrule, "\n")
     cat("Number of unique death times:    ", length(x$unique.death.times), "\n")
   }
   if (x$treetype == "Classification") {
