@@ -17,10 +17,11 @@
 # Written by:
 #
 #   Marvin N. Wright
-# Institut für Medizinische Biometrie und Statistik
-# Universität zu Lübeck
+# Institut fuer Medizinische Biometrie und Statistik
+# Universitaet zu Luebeck
 # Ratzeburger Allee 160
-# 23562 Lübeck
+# 23562 Luebeck
+# Germany
 #
 # http://www.imbs-luebeck.de
 # wright@imbs.uni-luebeck.de
@@ -32,11 +33,14 @@
 ##'
 ##' The tree type is determined by the type of the dependent variable.
 ##' For factors classification trees are grown, for numeric values regression trees and for survival objects survival trees.
-##' The Gini index is used as splitting rule for classification, the estimated response variances for regression and the log-rank test for survival.
+##' The Gini index is used as splitting rule for classification.
+##' For regression, the estimated response variances or maximally selected rank statistics (Wright et al. 2016) can be used.
+##' For Survival the log-rank test, a C-index based splitting rule (Schmid et al. 2015) and maximally selected rank statistics (Wright et al. 2016) are available.
 ##'
 ##' With the \code{probability} option and factor dependent variable a probability forest is grown.
-##' Here, the estimated response variances are used for splitting, as in regression forests.
+##' Here, the node impurity is used for splitting, as in classification forests.
 ##' Predictions are class probabilities for each sample.
+##' In contrast to other implementations, each tree returns a probability estimate and these estimates are averaged for the forest probability estimate.
 ##' For details see Malley et al. (2012).
 ##'
 ##' Note that for classification and regression nodes with size smaller than min.node.size can occur, like in original Random Forest.
@@ -46,6 +50,13 @@
 ##' Weights do not need to sum up to 1, they will be normalized later. 
 ##' The usage of \code{split.select.weights} can increase the computation times for large forests.
 ##'
+##' Unordered factor covariates can be handled in 3 different ways by using \code{respect.unordered.factors}: 
+##' For 'ignore' all factors are regarded ordered, for 'partition' all possible 2-partitions are considered for splitting. 
+##' For 'order' and 2-class classification the factor levels are ordered by their proportion falling in the second class, for regression by their mean response, as described in Hastie et al. (2009), chapter 9.2.4.
+##' For multiclass classification and survival outcomes, 'order' is experimental and should be used with care.
+##' The use of 'order' is recommended for 2-class classification and regression, as it computationally fast and can handle an unlimited number of factor levels. 
+##' Note that the factors are only reordered once and not again in each split. 
+##'
 ##' For a large number of variables and data frame as input data the formula interface can be slow or impossible to use.
 ##' Alternatively dependent.variable.name (and status.variable.name for survival) can be used.
 ##' Consider setting \code{save.memory = TRUE} if you encounter memory problems for very large datasets. 
@@ -56,9 +67,12 @@
 ##' To use only the SNPs without sex or other covariates from the phenotype file, use \code{0} on the right hand side of the formula. 
 ##' Note that missing values are treated as an extra category while splitting.
 ##' 
-##' Notes:
+##' See \url{https://github.com/imbs-hl/ranger} for the development version.
+##' 
+##' To use multithreading on Microsoft Windows platforms, there are currently two options:
 ##' \itemize{
-##'  \item Multithreading is currently not supported for Microsoft Windows platforms.
+##'  \item Use R-devel with the new toolchain
+##'  \item Install a binary version of ranger, download: \url{https://github.com/imbs-hl/ranger/releases}
 ##' }
 ##' 
 ##' @title Ranger
@@ -66,15 +80,22 @@
 ##' @param data Training data of class \code{data.frame}, \code{matrix} or \code{gwaa.data} (GenABEL).
 ##' @param num.trees Number of trees.
 ##' @param mtry Number of variables to possibly split at in each node. Default is the (rounded down) square root of the number variables. 
-##' @param importance Variable importance mode, one of 'none', 'impurity', 'permutation'. The 'impurity' measure is the Gini index for classification and the variance of the responses for regression.
+##' @param importance Variable importance mode, one of 'none', 'impurity', 'permutation'. The 'impurity' measure is the Gini index for classification and the variance of the responses for regression. For survival, only 'permutation' is available.
 ##' @param write.forest Save \code{ranger.forest} object, needed for prediction.
 ##' @param probability Grow a probability forest as in Malley et al. (2012). 
 ##' @param min.node.size Minimal node size. Default 1 for classification, 5 for regression, 3 for survival, and 10 for probability.
 ##' @param replace Sample with replacement. 
-##' @param split.select.weights Numeric vector with weights between 0 and 1, representing the probability to select variables for splitting.  
+##' @param sample.fraction Fraction of observations to sample. Default is 1 for sampling with replacement and 0.632 for sampling without replacement. 
+##' @param case.weights Weights for sampling of training observations. Observations with larger weights will be selected with higher probability in the bootstrap (or subsampled) samples for the trees.
+##' @param splitrule Splitting rule, regressiond and survival only. For regression one of "variance" or "maxstat" with default "variance". For survival "logrank", "C" or "maxstat" with default "logrank". 
+##' @param alpha For "maxstat" splitrule: Significance threshold to allow splitting.
+##' @param minprop For "maxstat" splitrule: Lower quantile of covariate distribtuion to be considered for splitting.
+##' @param split.select.weights Numeric vector with weights between 0 and 1, representing the probability to select variables for splitting. Alternatively, a list of size num.trees, containing split select weight vectors for each tree can be used.  
 ##' @param always.split.variables Character vector with variable names to be always tried for splitting.
-##' @param respect.unordered.factors Regard unordered factor covariates as unordered categorical variables. If \code{FALSE}, all factors are regarded ordered. 
+##' @param respect.unordered.factors Handling of unordered factor covariates. One of 'ignore', 'order' and 'partition' with default 'ignore'. Alternatively TRUE (='order') or FALSE (='ignore') can be used. See below for details. 
 ##' @param scale.permutation.importance Scale permutation importance by standard error as in (Breiman 2001). Only applicable if permutation variable importance mode selected.
+##' @param keep.inbag Save how often observations are in-bag in each tree. 
+##' @param holdout Hold-out mode. Hold-out all samples with case weight 0 and use these for variable importance and prediction error.
 ##' @param num.threads Number of threads. Default is number of CPUs available.
 ##' @param save.memory Use memory saving (but slower) splitting mode. No effect for GWAS data.
 ##' @param verbose Verbose output on or off.
@@ -83,25 +104,26 @@
 ##' @param status.variable.name Name of status variable, only applicable to survival data and needed if no formula given. Use 1 for event and 0 for censoring.
 ##' @param classification Only needed if data is a matrix. Set to \code{TRUE} to grow a classification forest.
 ##' @return Object of class \code{ranger} with elements
-##'   \tabular{ll}{
-##'       \code{forest} \tab Saved forest (If write.forest set to TRUE). Note that the variable IDs in the \code{split.varIDs} object do not necessarily represent the column number in R. \cr
-##'       \code{predictions}    \tab Predicted classes/values, based on out of bag samples (classification and regression only). \cr
-##'       \code{variable.importance}     \tab Variable importance for each independent variable. \cr
-##'       \code{prediction.error}   \tab Overall out of bag prediction error. For classification this is the fraction of missclassified samples, for regression the mean squared error and for survival one minus Harrell's c-index. \cr
-##'       \code{r.squared}   \tab R squared. Also called explained variance or coefficient of determination (regression only). \cr
-##'       \code{classification.table} \tab Contingency table for classes and predictions based on out of bag samples (classification only). \cr
-##'       \code{unique.death.times} \tab Unique death times (survival only). \cr
-##'       \code{chf} \tab Estimated cumulative hazard function for each sample (survival only). \cr
-##'       \code{survival} \tab Estimated survival function for each sample (survival only). \cr
-##'       \code{call}    \tab Function call. \cr
-##'       \code{num.trees}   \tab Number of trees. \cr
-##'       \code{num.independent.variables} \tab Number of independent variables. \cr
-##'       \code{mtry}    \tab Value of mtry used. \cr
-##'       \code{min.node.size}   \tab Value of minimal node size used. \cr
-##'       \code{treetype}    \tab Type of forest/tree. classification, regression or survival. \cr
-##'       \code{importance.mode}     \tab Importance mode used. \cr
-##'       \code{num.samples}     \tab Number of samples.
-##'   }
+##'   \item{\code{forest}}{Saved forest (If write.forest set to TRUE). Note that the variable IDs in the \code{split.varIDs} object do not necessarily represent the column number in R.}
+##'   \item{\code{predictions}}{Predicted classes/values, based on out of bag samples (classification and regression only).}
+##'   \item{\code{forest}}{Saved forest (If write.forest set to TRUE). Note that the variable IDs in the \code{split.varIDs} object do not necessarily represent the column number in R.}
+##'   \item{\code{predictions}}{Predicted classes/values, based on out of bag samples (classification and regression only).}
+##'   \item{\code{variable.importance}}{Variable importance for each independent variable.}
+##'   \item{\code{prediction.error}}{Overall out of bag prediction error. For classification this is the fraction of missclassified samples, for regression the mean squared error and for survival one minus Harrell's c-index.}
+##'   \item{\code{r.squared}}{R squared. Also called explained variance or coefficient of determination (regression only).}
+##'   \item{\code{confusion.matrix}}{Contingency table for classes and predictions based on out of bag samples (classification only).}
+##'   \item{\code{unique.death.times}}{Unique death times (survival only).}
+##'   \item{\code{chf}}{Estimated cumulative hazard function for each sample (survival only).}
+##'   \item{\code{survival}}{Estimated survival function for each sample (survival only).}
+##'   \item{\code{call}}{Function call.}
+##'   \item{\code{num.trees}}{Number of trees.}
+##'   \item{\code{num.independent.variables}}{Number of independent variables.}
+##'   \item{\code{mtry}}{Value of mtry used.}
+##'   \item{\code{min.node.size}}{Value of minimal node size used.}
+##'   \item{\code{treetype}}{Type of forest/tree. classification, regression or survival.}
+##'   \item{\code{importance.mode}}{Importance mode used.}
+##'   \item{\code{num.samples}}{Number of samples.}
+##'   \item{\code{inbag.counts}}{Number of times the observations are in-bag in the trees.}
 ##' @examples
 ##' require(ranger)
 ##'
@@ -140,26 +162,39 @@
 ##'
 ##' @author Marvin N. Wright
 ##' @references
-##'   Breiman, L. (2001). Random forests. Mach Learn, 45(1), 5-32. \cr
-##'   Ishwaran, H., Kogalur, U. B., Blackstone, E. H., & Lauer, M. S. (2008). Random survival forests. Ann Appl Stat, 841-860. \cr
-##'   Malley, J. D., Kruppa, J., Dasgupta, A., Malley, K. G., & Ziegler, A. (2012). Probability machines: consistent probability estimation using nonparametric learning machines. Methods Inf Med, 51(1), 74.
+##' \itemize{
+##'   \item Wright, M. N. & Ziegler, A. (2016). ranger: A Fast Implementation of Random Forests for High Dimensional Data in C++ and R. Journal of Statistical Software, in press. \url{http://arxiv.org/abs/1508.04409}.
+##'   \item Schmid, M., Wright, M. N. & Ziegler, A. (2015). On the Use of Harrell's C for Node Splitting in Random Survival Forests. Technical Report. \url{http://arxiv.org/abs/1507.03092}. 
+##'   \item Wright, M. N., Dankowski, T. & Ziegler, A. (2016). Random forests for survival analysis using maximally selected rank statistics. Technical Report. \url{http://arxiv.org/abs/1605.03391}.
+##' 
+##'   \item Breiman, L. (2001). Random forests. Mach Learn, 45(1), 5-32. 
+##'   \item Ishwaran, H., Kogalur, U. B., Blackstone, E. H., & Lauer, M. S. (2008). Random survival forests. Ann Appl Stat, 841-860. 
+##'   \item Malley, J. D., Kruppa, J., Dasgupta, A., Malley, K. G., & Ziegler, A. (2012). Probability machines: consistent probability estimation using nonparametric learning machines. Methods Inf Med, 51(1), 74.
+##'   \item Hastie, T., Tibshirani, R., Friedman, J. (2009). The Elements of Statistical Learning. Springer, New York. 2nd edition.
+##'   }
 ##' @seealso \code{\link{predict.ranger}}
 ##' @useDynLib ranger
 ##' @importFrom Rcpp evalCpp
+##' @import stats 
+##' @import utils
 ##' @export
 ranger <- function(formula = NULL, data = NULL, num.trees = 500, mtry = NULL,
                    importance = "none", write.forest = FALSE, probability = FALSE,
-                   min.node.size = NULL, replace = TRUE,
+                   min.node.size = NULL, replace = TRUE, 
+                   sample.fraction = ifelse(replace, 1, 0.632), 
+                   case.weights = NULL, 
+                   splitrule = NULL, alpha = 0.5, minprop = 0.1,
                    split.select.weights = NULL, always.split.variables = NULL,
-                   respect.unordered.factors = FALSE,
+                   respect.unordered.factors = "ignore",
                    scale.permutation.importance = FALSE,
+                   keep.inbag = FALSE, holdout = FALSE,
                    num.threads = NULL, save.memory = FALSE,
                    verbose = TRUE, seed = NULL, 
                    dependent.variable.name = NULL, status.variable.name = NULL, 
                    classification = NULL) {
   
   ## GenABEL GWA data
-  if (class(data) == "gwaa.data") {
+  if ("gwaa.data" %in% class(data)) {
     snp.names <- data@gtdata@snpnames
     sparse.data <- data@gtdata@gtps@.Data
     data <- data@phdata
@@ -194,11 +229,6 @@ ranger <- function(formula = NULL, data = NULL, num.trees = 500, mtry = NULL,
     response <- data.selected[[1]]
   }
   
-  ## Probability estimation
-  if (probability & !is.factor(response)) {
-    stop("Error: Probability estimation is only applicable to categorical (factor) dependent variables.")
-  }
-  
   ## Treetype
   if (is.factor(response)) {
     if (probability) {
@@ -209,10 +239,12 @@ ranger <- function(formula = NULL, data = NULL, num.trees = 500, mtry = NULL,
   } else if (is.numeric(response) & is.vector(response)) {
     if (!is.null(classification) && classification) {
       treetype <- 1
+    } else if (probability) {
+      treetype <- 9
     } else {
       treetype <- 3
     }
-  } else if (class(response) == "Surv" | class(response) == "data.frame") {
+  } else if (class(response) == "Surv" | is.data.frame(response) | is.matrix(response)) {
     treetype <- 5
   } else {
     stop("Error: Unsupported type of dependent variable.")
@@ -233,17 +265,64 @@ ranger <- function(formula = NULL, data = NULL, num.trees = 500, mtry = NULL,
                                                           colnames(data.selected) != status.variable.name]
   }
   
-  ## Input data and variable names
+  ## Old version of if respect.unordered.factors
+  if (respect.unordered.factors == TRUE) {
+    respect.unordered.factors <- "order"
+  } else if (respect.unordered.factors == FALSE) {
+    respect.unordered.factors <- "ignore"
+  }
+  
+  ## Recode characters as factors and recode factors if 'order' mode
+  if (!is.matrix(data.selected)) {
+    character.idx <- sapply(data.selected, is.character)
+    
+    if (respect.unordered.factors == "order") {
+      ## Recode characters and unordered factors
+      names.selected <- names(data.selected)
+      ordered.idx <- sapply(data.selected, is.ordered)
+      factor.idx <- sapply(data.selected, is.factor)
+      independent.idx <- names.selected != dependent.variable.name & 
+        names.selected != status.variable.name & 
+        names.selected != paste0("Surv(", dependent.variable.name, ", ", status.variable.name, ")")
+      recode.idx <- independent.idx & (character.idx | (factor.idx & !ordered.idx))
+
+      ## Numeric response
+      if (is.factor(response)) {
+        num.response <- as.numeric(response)
+      } else if (!is.null(dim(response))) {
+        num.response <- response[, 1]
+      } else {
+        num.response <- response
+      }
+
+      ## Recode each column
+      data.selected[recode.idx] <- lapply(data.selected[recode.idx], function(x) {
+        ## Order factor levels
+        means <- aggregate(num.response~x, FUN=mean)
+        levels.ordered <- means$x[order(means$num.response)]
+        
+        ## Return reordered factor
+        factor(x, levels = levels.ordered)
+      })
+      
+      ## Save levels
+      covariate.levels <- lapply(data.selected[independent.idx], levels)
+    } else {
+      ## Recode characters only
+      data.selected[character.idx] <- lapply(data.selected[character.idx], factor)
+    }
+  }
+  
+  ## Input data and variable names, create final data matrix
   if (!is.null(formula) & treetype == 5) {
-    data.final <- cbind(response[, 1], response[, 2],
-                        data.selected[-1])
+    data.final <- data.matrix(cbind(response[, 1], response[, 2],
+                              data.selected[-1]))
     colnames(data.final) <- c(dependent.variable.name, status.variable.name,
                               independent.variable.names)
-  } else {
+  } else if (is.matrix(data.selected)) {
     data.final <- data.selected
-  }
-  if (!is.matrix(data.selected)) {
-    data.final <- data.matrix(data.final)
+  } else {
+    data.final <- data.matrix(data.selected)
   }
   variable.names <- colnames(data.final)
   
@@ -273,6 +352,11 @@ ranger <- function(formula = NULL, data = NULL, num.trees = 500, mtry = NULL,
     seed <- runif(1 , 0, .Machine$integer.max)
   }
   
+  ## Keep inbag
+  if (!is.logical(keep.inbag)) {
+    stop("Error: Invalid value for keep.inbag")
+  }
+  
   ## Num threads
   ## Default 0 -> detect from system in C++.
   if (is.null(num.threads)) {
@@ -288,6 +372,11 @@ ranger <- function(formula = NULL, data = NULL, num.trees = 500, mtry = NULL,
     stop("Error: Invalid value for min.node.size")
   }
   
+  ## Sample fraction
+  if (!is.numeric(sample.fraction) | sample.fraction <= 0 | sample.fraction > 1) {
+    stop("Error: Invalid value for sample.fraction. Please give a value in (0,1].")
+  }
+  
   ## Importance mode
   if (is.null(importance) | importance == "none") {
     importance.mode <- 0
@@ -306,12 +395,43 @@ ranger <- function(formula = NULL, data = NULL, num.trees = 500, mtry = NULL,
     stop("Error: Unknown importance mode.")
   }
   
+  ## Case weights: NULL for no weights
+  if (is.null(case.weights)) {
+    case.weights <- c(0,0)
+    use.case.weights <- FALSE
+    if (holdout) {
+      stop("Error: Case weights required to use holdout mode.")
+    }
+  } else {
+    use.case.weights <- TRUE
+    
+    ## Sample from non-zero weights in holdout mode
+    if (holdout) {
+      sample.fraction <- sample.fraction * mean(case.weights > 0)
+    }
+    
+    if (!replace && sum(case.weights > 0) < sample.fraction * nrow(data.final)) {
+      stop("Error: Fewer non-zero case weights than observations to sample.")
+    }
+  }
+  
   ## Split select weights: NULL for no weights
   if (is.null(split.select.weights)) {
-    split.select.weights <- c(0,0)
+    split.select.weights <- list(c(0,0))
     use.split.select.weights <- FALSE
-  } else {
+  } else if (is.numeric(split.select.weights)) {
+    if (length(split.select.weights) != length(all.independent.variable.names)) {
+      stop("Error: Number of split select weights not equal to number of independent variables.")
+    }
+    split.select.weights <- list(split.select.weights)
     use.split.select.weights <- TRUE
+  } else if (is.list(split.select.weights)) {
+    if (length(split.select.weights) != num.trees) {
+      stop("Error: Size of split select weights list not equal to number of trees.")
+    }
+    use.split.select.weights <- TRUE
+  } else {
+    stop("Error: Invalid split select weights.")
   }
   
   ## Always split variables: NULL for no variables
@@ -323,11 +443,61 @@ ranger <- function(formula = NULL, data = NULL, num.trees = 500, mtry = NULL,
   }
   
   if (use.split.select.weights & use.always.split.variables) {
-    stop("Error: Please use only one option of use.split.select.weights and use.always.split.variables.")
+    stop("Error: Please use only one option of split.select.weights and always.split.variables.")
   }
   
+  ## Splitting rule
+  if (is.null(splitrule)) {
+    if (treetype == 5) {
+      splitrule <- "logrank"
+    } else if (treetype == 3) {
+      splitrule <- "variance"
+    }
+    splitrule.num <- 1
+  } else if (splitrule == "logrank") {
+    if (treetype == 5) {
+      splitrule.num <- 1
+    } else {
+      stop("Error: logrank splitrule applicable to survival data only.")
+    }
+  } else if (splitrule == "variance") {
+    if (treetype == 3) {
+      splitrule.num <- 2
+    } else {
+      stop("Error: variance splitrule applicable to regression data only.")
+    }
+  } else if (splitrule == "auc" | splitrule == "C") {
+    if (treetype == 5) {
+      splitrule.num <- 2
+    } else {
+      stop("Error: C index splitrule applicable to survival data only.")
+    }
+  } else if (splitrule == "auc_ignore_ties" | splitrule == "C_ignore_ties") {
+    if (treetype == 5) {
+      splitrule.num <- 3
+    } else {
+      stop("Error: C index splitrule applicable to survival data only.")
+    }
+  } else if (splitrule == "maxstat") {
+    if (treetype == 5 | treetype == 3) {
+      splitrule.num <- 4
+    } else {
+      stop("Error: maxstat splitrule applicable to regression or survival data only.")
+    }
+  } else {
+    stop("Error: Unknown splitrule.")
+  }
+  
+  ## Maxstat splitting
+  if (alpha < 0 | alpha > 1) {
+    stop("Error: Invalid value for alpha, please give a value between 0 and 1.")
+  }
+  if (minprop < 0 | minprop > 0.5) {
+    stop("Error: Invalid value for minprop, please give a value between 0 and 0.5.")
+  }
+
   ## Unordered factors  
-  if (respect.unordered.factors) {
+  if (respect.unordered.factors == "partition") {
     names.selected <- names(data.selected)
     ordered.idx <- sapply(data.selected, is.ordered)
     factor.idx <- sapply(data.selected, is.factor)
@@ -341,19 +511,45 @@ ranger <- function(formula = NULL, data = NULL, num.trees = 500, mtry = NULL,
       max.level.count <- 8*.Machine$sizeof.pointer - 1
       if (max(num.levels) > max.level.count) {
         stop(paste("Too many levels in unordered categorical variable ", unordered.factor.variables[which.max(num.levels)], 
-                   ". Only ", max.level.count, " levels allowed on this system. Consider ordering this factor.", sep = ""))
+                   ". Only ", max.level.count, " levels allowed on this system. Consider using the 'order' option.", sep = ""))
       } 
     } else {
       unordered.factor.variables <- c("0", "0")
       use.unordered.factor.variables <- FALSE
     } 
-  } else {
+  } else if (respect.unordered.factors == "ignore" | respect.unordered.factors == "order") {
+    ## Ordering for "order" is handled above
     unordered.factor.variables <- c("0", "0")
     use.unordered.factor.variables <- FALSE
+  } else {
+    stop("Error: Invalid value for respect.unordered.factors, please use 'order', 'partition' or 'ignore'.")
+  }
+
+  ## Unordered maxstat splitting not possible
+  if (use.unordered.factor.variables & !is.null(splitrule)) {
+    if (splitrule == "maxstat") {
+      stop("Error: Unordered factor splitting not implemented for 'maxstat' splitting rule.")
+    } else if (splitrule %in% c("C", "auc", "C_ignore_ties", "auc_ignore_ties")) {
+      stop("Error: Unordered factor splitting not implemented for 'C' splitting rule.")
+    }
   }
   
+  ## Warning for experimental 'order' splitting 
+  if (respect.unordered.factors == "order") {
+    if (treetype == 5) {
+      warning("Warning: The 'order' mode for unordered factor handling for survival outcomes is experimental.")
+    } else if (treetype == 1 | treetype == 9) {
+      if (nlevels(response) > 2) {
+        warning("Warning: The 'order' mode for unordered factor handling for multiclass classification is experimental.")
+      }
+    } else if (treetype == 3 & splitrule == "maxstat") {
+      warning("Warning: The 'order' mode for unordered factor handling with the 'maxstat' splitrule is experimental.")
+    }
+  }
+
   ## Prediction mode always false. Use predict.ranger() method.
   prediction.mode <- FALSE
+  predict.all <- FALSE
   
   ## No loaded forest object
   loaded.forest <- list()
@@ -368,10 +564,11 @@ ranger <- function(formula = NULL, data = NULL, num.trees = 500, mtry = NULL,
                       always.split.variables, use.always.split.variables,
                       status.variable.name, prediction.mode, loaded.forest, sparse.data,
                       replace, probability, unordered.factor.variables, use.unordered.factor.variables, 
-                      save.memory)
+                      save.memory, splitrule.num, case.weights, use.case.weights, predict.all, 
+                      keep.inbag, sample.fraction, alpha, minprop, holdout)
   
   if (length(result) == 0) {
-    stop("Internal error.")
+    stop("User interrupt or internal error.")
   }
   
   ## Prepare results
@@ -384,13 +581,18 @@ ranger <- function(formula = NULL, data = NULL, num.trees = 500, mtry = NULL,
   if (treetype == 1 & is.factor(response)) {
     result$predictions <- factor(result$predictions, levels = 1:nlevels(response),
                                  labels = levels(response))
-    result$classification.table <- table(result$predictions, unlist(data[, dependent.variable.name]), dnn = c("predicted", "true"))
+    result$confusion.matrix <- table(unlist(data.final[, dependent.variable.name]), result$predictions, dnn = c("true", "predicted"))
   } else if (treetype == 5) {
     result$chf <- result$predictions
     result$predictions <- NULL
     result$survival <- exp(-result$chf)
   } else if (treetype == 9 & !is.matrix(data)) {
-    colnames(result$predictions) <- levels(response)
+    colnames(result$predictions) <- levels(droplevels(response))
+  }
+  
+  ## Splitrule
+  if (treetype == 3 | treetype == 5) {
+    result$splitrule <- splitrule
   }
   
   ## Set treetype
@@ -416,6 +618,11 @@ ranger <- function(formula = NULL, data = NULL, num.trees = 500, mtry = NULL,
     result$forest$independent.variable.names <- independent.variable.names
     result$forest$treetype <- result$treetype
     class(result$forest) <- "ranger.forest"
+    
+    ## In 'ordered' mode, save covariate levels
+    if (respect.unordered.factors == "order" & !is.matrix(data)) {
+      result$forest$covariate.levels <- covariate.levels
+    }
   }
   
   class(result) <- "ranger"
