@@ -50,21 +50,20 @@ public:
       std::vector<size_t>* deterministic_varIDs, std::vector<size_t>* split_select_varIDs,
       std::vector<double>* split_select_weights, ImportanceMode importance_mode, uint min_node_size,
       std::vector<size_t>* no_split_variables, bool sample_with_replacement, std::vector<bool>* is_unordered,
-      bool memory_saving_splitting);
+      bool memory_saving_splitting, SplitRule splitrule, std::vector<double>* case_weights, bool keep_inbag,
+      double sample_fraction, double alpha, double minprop, bool holdout);
+
   virtual void initInternal() = 0;
 
-  void grow();
+  void grow(std::vector<double>* variable_importance);
 
   void predict(const Data* prediction_data, bool oob_prediction);
 
-  void computePermutationImportance();
+  void computePermutationImportance(std::vector<double>* forest_importance, std::vector<double>* forest_variance);
 
   void appendToFile(std::ofstream& file);
   virtual void appendToFileInternal(std::ofstream& file) = 0;
 
-  const std::vector<double>& getVariableImportance() const {
-    return variable_importance;
-  }
   const std::vector<std::vector<size_t> >& getChildNodeIDs() const {
     return child_nodeIDs;
   }
@@ -82,6 +81,10 @@ public:
     return num_samples_oob;
   }
 
+  const std::vector<size_t>& getInbagCounts() const {
+    return inbag_counts;
+  }
+
 protected:
   void createPossibleSplitVarSubset(std::vector<size_t>& result);
 
@@ -99,6 +102,9 @@ protected:
   void bootstrap();
   void bootstrapWithoutReplacement();
 
+  void bootstrapWeighted();
+  void bootstrapWithoutReplacementWeighted();
+
   virtual void cleanUpInternal() = 0;
 
   size_t dependent_varID;
@@ -125,6 +131,9 @@ protected:
   std::vector<size_t>* split_select_varIDs;
   std::vector<double>* split_select_weights;
 
+  // Bootstrap weights
+  std::vector<double>* case_weights;
+
   // Splitting variable for each node
   std::vector<size_t> split_varIDs;
 
@@ -132,7 +141,7 @@ protected:
   // For terminal nodes the prediction value is saved here
   std::vector<double> split_values;
 
-  // For each node a vector with child node IDs, for binary trees second dim = 2
+  // Vector of left and right child node IDs, 0 for no child
   std::vector<std::vector<size_t>> child_nodeIDs;
 
   // For each node a vector with IDs of samples in node
@@ -141,6 +150,13 @@ protected:
   // IDs of OOB individuals, sorted
   std::vector<size_t> oob_sampleIDs;
 
+  // Holdout mode
+  bool holdout;
+
+  // Inbag counts
+  bool keep_inbag;
+  std::vector<size_t> inbag_counts;
+
   // Random number generator
   std::mt19937_64 random_number_generator;
 
@@ -148,7 +164,7 @@ protected:
   Data* data;
 
   // Variable importance for all variables
-  std::vector<double> variable_importance;
+  std::vector<double>* variable_importance;
   ImportanceMode importance_mode;
 
   // When growing here the OOB set is used
@@ -156,7 +172,12 @@ protected:
   std::vector<size_t> prediction_terminal_nodeIDs;
 
   bool sample_with_replacement;
+  double sample_fraction;
+
   bool memory_saving_splitting;
+  SplitRule splitrule;
+  double alpha;
+  double minprop;
 
 private:
   DISALLOW_COPY_AND_ASSIGN(Tree);
