@@ -58,20 +58,34 @@ public:
   }
 
 private:
-  bool splitNodeInternal(size_t nodeID, std::vector<size_t>& possible_split_varIDs);
-  void createEmptyNodeInternal();
 
+  void createEmptyNodeInternal();
+  void computeSurvival(size_t nodeID);
   double computePredictionAccuracyInternal();
 
-  // Called by splitNodeInternal(). Sets split_varIDs and split_values.
-  bool findBestSplitLogRank(size_t nodeID, std::vector<size_t>& possible_split_varIDs);
+  bool splitNodeInternal(size_t nodeID, std::vector<size_t>& possible_split_varIDs);
+
+  bool findBestSplit(size_t nodeID, std::vector<size_t>& possible_split_varIDs);
+  bool findBestSplitMaxstat(size_t nodeID, std::vector<size_t>& possible_split_varIDs);
+
+  void findBestSplitValueLogRank(size_t nodeID, size_t varID, std::vector<double>& possible_split_values,
+      double& best_value, size_t& best_varID, double& best_logrank);
+  void findBestSplitValueLogRankUnordered(size_t nodeID, size_t varID, std::vector<double>& factor_levels,
+      double& best_value, size_t& best_varID, double& best_logrank);
+  void findBestSplitValueAUC(size_t nodeID, size_t varID,
+      double& best_value, size_t& best_varID, double& best_auc);
+
   void computeDeathCounts(size_t nodeID);
   void computeChildDeathCounts(size_t nodeID, size_t varID, std::vector<double>& possible_split_values,
       size_t* num_samples_right_child, size_t* num_samples_at_risk_right_child, size_t* num_deaths_right_child);
-  void findBestSplitValueLogRank(size_t nodeID, size_t varID, std::vector<double>& possible_split_values,
-      double& best_value, size_t& best_varID, double& best_logrank);
-  void findBestSplitValueLogRankUnordered(size_t nodeID, size_t varID, std::vector<double>& factor_levels, double& best_value,
-      size_t& best_varID, double& best_logrank);
+
+  void computeAucSplit(double time_k, double time_l, double status_k, double status_l, double value_k, double value_l,
+      size_t num_splits, std::vector<double>& possible_split_values, double* num_count, double* num_total);
+
+  void findBestSplitValueLogRank(size_t nodeID, size_t varID, double& best_value, size_t& best_varID,
+      double& best_logrank);
+  void findBestSplitValueLogRankUnordered(size_t nodeID, size_t varID, double& best_value, size_t& best_varID,
+      double& best_logrank);
 
   void cleanUpInternal() {
     delete[] num_deaths;
