@@ -1,6 +1,6 @@
 /*-------------------------------------------------------------------------------
 This file is part of Ranger.
-    
+
 Ranger is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
@@ -14,13 +14,13 @@ GNU General Public License for more details.
 You should have received a copy of the GNU General Public License
 along with Ranger. If not, see <http://www.gnu.org/licenses/>.
 
-Written by: 
+Written by:
 
 Marvin N. Wright
 Institut für Medizinische Biometrie und Statistik
 Universität zu Lübeck
 Ratzeburger Allee 160
-23562 Lübeck 
+23562 Lübeck
 
 http://www.imbs-luebeck.de
 wright@imbs.uni-luebeck.de
@@ -33,6 +33,20 @@ wright@imbs.uni-luebeck.de
     TypeName(const TypeName&);             \
     void operator=(const TypeName&)
 
+// Pi
+#ifndef M_PI
+#define M_PI 3.14159265358979323846
+#endif
+
+// Old/new Win build
+#ifdef WIN_R_BUILD
+  #if __cplusplus < 201103L
+    #define OLD_WIN_R_BUILD
+  #else
+    #define NEW_WIN_R_BUILD
+  #endif
+#endif
+
 typedef unsigned int uint;
 
 // Tree types, probability is not selected by ID
@@ -65,6 +79,14 @@ enum ImportanceMode {
 };
 const uint MAX_IMP_MODE = 4;
 
+// Split mode
+enum SplitRule {
+  LOGRANK = 1,
+  AUC = 2,
+  AUC_IGNORE_TIES = 3,
+  MAXSTAT = 4
+};
+
 // Default values
 const uint DEFAULT_NUM_TREE = 500;
 const uint DEFAULT_NUM_THREADS = 0;
@@ -75,7 +97,14 @@ const uint DEFAULT_MIN_NODE_SIZE_REGRESSION = 5;
 const uint DEFAULT_MIN_NODE_SIZE_SURVIVAL = 3;
 const uint DEFAULT_MIN_NODE_SIZE_PROBABILITY = 10;
 
+const SplitRule DEFAULT_SPLITRULE = LOGRANK;
+const double DEFAULT_ALPHA = 0.5;
+const double DEFAULT_MINPROP = 0.1;
+
 // Interval to print progress in seconds
 const double STATUS_INTERVAL = 30.0;
 
+// Threshold for q value split method switch
+const double Q_THRESHOLD = 0.02;
+
 #endif /* GLOBALS_H_ */
